use Config;
use File::Basename qw(basename dirname);
chdir(dirname($0));

$perl = shift(@ARGV);

$use = "use lib qw(\n\t" . join("\n\t", @ARGV) . ");\n";

($file = basename($0)) =~ s/\.PL$//;
$file =~ s/\.pl$//
	if ($Config{'osname'} eq 'VMS' or
	    $Config{'osname'} eq 'OS2');  # "case-forgiving"
open OUT,">$file" or die "Can't create $file: $!";
chmod(0755, $file);
print "Extracting $file (with variable substitutions)\n";

print OUT <<"!GROK!THIS!";
#!$perl -w

#
# Warning: File automatically generated. Any changes made here
# will be lost.
#

$use

!GROK!THIS!
print OUT <<'!NO!SUBS!';

=head1 NAME

add-asset

=head1 SYNOPSIS

add-asset asset_url

=head1 DESCRIPTION

See http://nhse.mcs.anl.gov/NHSE96/RIB/MTk/tools/add-asset.htm
for a full description of this command.

=cut

use strict;

use Getopt::Long;

use MTk;

eval {
    &main;
};

if ($@)
{
    print "Add-asset failed:\n\t$@\n";
    exit(1);
}


exit(0);

sub main
{
    my($asset_url, $db, $repos_name, @assets);
    my(@files, $debug, $opt_rc);

    $opt_rc = &GetOptions("f=s@" => \@files,
			  "d" => \$debug);

    if ($opt_rc == 0)
    {
	&usage;
    }

    if (@ARGV == 0)
    {
	&usage;
    }

    $repos_name = shift(@ARGV);

    if (@ARGV > 0)
    {
	@assets = @ARGV;
    }
    elsif (@files == 0)
    {
	&usage;
    }

    if (@files)
    {
	my($file);
	for $file (@files)
	{
	    load_asset_file($file, \@assets);
	}
    }

    $db = $MTk::DBClass->new();
    $db or die "Cannot connect to database\n";
    
    $db->open();

    &validate_repository($db, $repos_name);

    for $asset_url (@assets)
    {
	if ($debug)
	{
	    print "Would add $asset_url\n";
	}
	else
	{
	    eval {
		&add_asset($db, $repos_name, $asset_url);
	    };
	    if ($@)
	    {
		print "Add-asset $asset_url failed: $@";
	    }
	}
    }
    #
    # This is needed to avoid a warning about invalidated cursors with dbi.
    # I suspect a bug somewhere in perl itself...
    
    1;
}

sub usage
{
    die "Usage: $0 [-f asset-file] repository [assets]\n";
}

sub load_asset_file
{
    my($file, $asset_list) = @_;
    my($fh);

    $fh = new FileHandle("<$file");
    $fh or die "Cannot open asset file $file: $!\n";
    while (<$fh>)
    {
	chop;
	s/^\s+//;
	s/\s+$//;
	next if /^#/;
	next if /\s/;
	next if $_ eq '';
	push(@$asset_list, $_);
    }
    $fh->close();
}

#
# Make sure that this is a valid repository.
#

sub validate_repository
{
    my($db, $repos_name) = @_;
    my($q, $official_name);

    $q = $db->query("select official_name from Repository where name = '$repos_name'");
    $q or die "Repository $repos_name not found\n";

    $official_name = $q->item(0, 0);
    print "Adding to repository $official_name\n";
}


#
# Add this asset to the database.
# 

sub add_asset
{
    my($db, $repos_name, $asset_url) = @_;
    my($asset, $rc);
    my($q);

    $asset = MTk::DBTable::Asset->new($asset_url, $db);
    
    #
    # First see if it's already there
    #

    $rc = $asset->fetch();

    if ($rc)
    {
	die "Asset $asset_url already present\n";
    }
    print "adding $asset_url\n";

    $asset->field_value('repository', $repos_name);
    $asset->field_value('expires', 4);
    $asset->field_value('last_retrieval', -1);
    $asset->field_value('last_modified', -1);

    $asset->store();
}
!NO!SUBS!
    
