use Config;
use File::Basename qw(basename dirname);
chdir(dirname($0));

$perl = shift(@ARGV);

$use = "use lib qw(\n\t" . join("\n\t", @ARGV) . ");\n";

($file = basename($0)) =~ s/\.PL$//;
$file =~ s/\.pl$//
	if ($Config{'osname'} eq 'VMS' or
	    $Config{'osname'} eq 'OS2');  # "case-forgiving"
open OUT,">$file" or die "Can't create $file: $!";
chmod(0755, $file);
print "Extracting $file (with variable substitutions)\n";

print OUT <<"!GROK!THIS!";
#!$perl -w

#
# Warning: File automatically generated. Any changes made here
# will be lost.
#

$use

!GROK!THIS!
print OUT <<'!NO!SUBS!';
=head1 Name

init-repository

=head1 SYNOPSIS

init-repository 

=head1 DESCRIPTION

See http://nhse.mcs.anl.gov/NHSE96/RIB/MTk/tools/init-repository.htm
for a full description of this command.

=cut

use strict;

use MTk;
use FileHandle;
use File::Path;

eval {
    &main;
};

if ($@)
{
    print "Repository initialization failed:\n\t$@\n";
    exit(1);
}


exit(0);

sub main
{
    my($repos_info);

    if (@ARGV == 1)
    {
	$repos_info = &parse_config_file($ARGV[0]);
    }
    else
    {
	$repos_info = &prompt_user;
    }

    &check_user_input($repos_info);

    &init_repository($repos_info);
}

#
# Guts of repository initialization. Note that we are using
# eval/die exception handling here, so we can safely die with an
# error message if something is wrong.
#

sub init_repository
{
    my($repos_info) = @_;
    my($db, $ok);

    $db = $MTk::DBClass->new();
    $db or die "Cannot connect to database\n";
    
    $db->open();
    
    &check_existing_db($db);
    &create_tables($db);
    &initialize_tables($db, $repos_info);
    &create_directory_structure($repos_info);
}

#
# Check to see if there are already repository tables in this database.
# If there are, bail.
#

sub check_existing_db
{
    my($db) = @_;
    my($table);

    for $table (&MTk::DBTable::table_names())
    {
	if ($db->table_exists($table))
	{
	    die "Table `$table' already exists in database\n";
	}
    }
}

#
# Create the tables for the repository, using the information in
# MTk::DBTable
#

sub create_tables
{
    my($db) = @_;
    my($table, $q, $cl);

    for $table (&MTk::DBTable::table_names())
    {
	$cl = new MTk::DBTable($table);
	$cl or die "Cannot find definition for table $table\n";

	$q = $db->query($cl->create_clause());
    }
}

#
# Create the repository directory structure:
#
# base_path/
#           cgi-bin/
#           README
#           index.html
#

sub create_directory_structure
{
    my($repos_info) = @_;
    my($dir, $fh);

    $dir = $repos_info->{base_path};
    
    if (! -d $dir)
    {
	mkpath($dir, 1, 0775);
    }

    if (! -d "$dir/cgi-bin")
    {
	mkpath("$dir/cgi-bin", 1, 0775);
    }

    if (-f "$dir/index.html")
    {
	print "Skipping creation of index page\n";
    }
    else
    {
	$fh = new FileHandle ">$dir/index.html";
	$fh or die "Cannot create $dir/index.html: $!\n";

	print $fh <<END;
<html>
<head>
<title>Welcome to $repos_info->{db_long_name}</title>
</head>
<body>
<h1>Welcome to $repos_info->{db_long_name}</h1>
It isn't ready to use yet, tho.
</body>
</html>
END
        $fh->close();
    }

    if (-f "$dir/README")
    {
	print "Skipping creation of README\n";
    }
    else
    {
        $fh = new FileHandle ">$dir/README";
        $fh or die "Cannot create $dir/README: $!\n";

        print $fh <<END;
Welcome to $repos_info->{db_long_name}

It isn't ready to use yet, tho.

If you're using NCSA httpd, you may need to add the following to
your srm.conf configuration file:

	Alias $repos_info->{base_url}/ $repos_info->{base_path}/
	ScriptAlias $repos_info->{base_url}/cgi-bin/ $repos_info->{base_path}/cgi-bin/
END
        $fh->close();
    }
}

#
# Fill in the repository table with the information from the
# user.
#

sub initialize_tables
{
    my($db, $repos_info) = @_;
    my($q);

    $q = $db->query(<<END);
	INSERT INTO Repository
	VALUES ('$repos_info->{db_name}',
		'$repos_info->{db_long_name}',
		'$repos_info->{base_path}',
		'$repos_info->{base_url}')
END
}

sub parse_config_file
{
    my($file) = @_;
    my($fp, $name, $val, $repos_info);

    $repos_info = {};

    $fp = new FileHandle "<$file";
    $fp or die "Cannot open configuration file $file\n";

    while (<$fp>)
    {
	s/^\s+//;
	s/\s+$//;
	next if /^#/;
	($name, $val) = split(/\s+/, $_, 2);
	$repos_info->{$name} = $val;
    }
    return $repos_info;
}

sub prompt_user
{
    my($repos_info);

    $repos_info = {};

    print $MTk::Strings::splash_text;

    print "Repository full name: ";
    chop($repos_info->{db_long_name} = <STDIN>);

    print "\n";

    while (1)
    {
	print "Repository short name: ";
	chop($repos_info->{db_name} = <STDIN>);
	$repos_info->{db_name} =~ s/^\s+//;
	$repos_info->{db_name} =~ s/\s+$//;
	if ($repos_info->{db_name} =~ /\s/)
	{
	    print "The short name must not contain spaces\n";
	    next;
	}
	else
	{
	    last;
	}
    }

    print "Base directory for repository web tree: ";
    chop($repos_info->{base_path} = <STDIN>);
    
    print "URL corresponding to the path $repos_info->{base_path}: ";
    chop($repos_info->{base_url} = <STDIN>);

    return $repos_info;
}

#
# Make sure we got all we need from the user
#

sub check_user_input
{
    my($repos_info) = @_;

    $repos_info->{db_long_name} ne ''
	or die "Missing repository long name\n";

    $repos_info->{db_name} ne ''
	or die "Missing repository name\n";

    $repos_info->{base_path} =~ m,^/, 
	or die "Repository base path must be a fully qualified pathname\n";

    $repos_info->{base_url} =~ m,^http://, 
	or die "Repository base url must be an absolute URL\n";
}
!NO!SUBS!

