package BIDM::Relationship;

=head1 NAME

BIDM::Relationship

=head1 DESCRIPTION

=cut

use strict;


use BIDM::MemberMixin;
use vars qw(@ISA);

@ISA = qw(BIDM::MemberMixin);


sub new
{
    my($class, $name) = @_;
    my($self);

    $self = {
	name => $name,
	description => '',
	is_optional => 0,
	is_single => 0,
	constraints => '',
	data_type => '',
    };
    bless($self, $class);

    return $self;
}

=head2 @related_objs = $rel->get_object_relationship($obj)

Follow this relationship on object $obj; that is, return the list of objects
which are related to $obj by this relationship.

=cut

sub get_object_relationship
{
    my($self, $obj) = @_;
    my($name, $hash);

    $name = $self->name();

    if (defined($hash = $obj->relationships()->{$name}))
    {
#	print "get_obj_rel: got hash=$hash\n";
#	my($k, $v);
#	while (($k, $v) = each(%$hash))
#	{
#	    print "   k=$k v=$v\n";
#	}
	my @values = values(%$hash);
	if (wantarray)
	{
	    return @values;
	}
	else
	{
	    return $values[0];
	}
    }
    else
    {
	return wantarray ? () : undef;
    }
}

sub load_from_fh
{
    my($self, $fh) = @_;
    my($key, $val);
    
    while (<$fh>)
    {
	chop;
	s/^\s+//;
	s/\s+$//;
	next if /^#/;
	last if /^%end/;

	($key, $val) = split(/\s+/, $_, 2);

	if (!defined($self->{$key}))
	{
	    die "Invalid relationship $key\n";
	}
#	print "   $key = $val\n";
	$self->$key($val);
    }
}

sub name { shift->_elem('name', @_); }
sub type { shift->_elem('type', @_); }
sub description { shift->_elem('description', @_); }
sub is_optional { shift->_elem('is_optional', @_); }
sub is_single { shift->_elem('is_single', @_); }
sub constraints { shift->_elem('constraints', @_); }
sub data_type { shift->_elem('data_type', @_); }

1;
